/*
 * Copyright (c) 2024.
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */

package de.ozgcloud.nachrichten.antragraum;

import de.ozgcloud.nachrichten.postfach.PostfachNachricht;
import de.ozgcloud.nachrichten.postfach.PostfachNachrichtTestFactory;
import io.grpc.stub.StreamObserver;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;

import java.util.UUID;
import java.util.stream.Stream;

import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

class AntragraumGrpcServiceTest {
	@Spy
	@InjectMocks
	private AntragraumGrpcService antragsraumGrpcService;
	@Mock
	private AntragraumService antragraumService;
	@Mock
	private AntragraumNachrichtMapper mapper;

	@Nested
	class TestFindRueckfragen {

		@Nested
		class TestFindRueckfragenGrpc {
			@Mock
			private StreamObserver<GrpcFindRueckfragenResponse> streamObserver;

			@BeforeEach
			void setup() {
				when(antragraumService.findRueckfragen(any())).thenReturn(Stream.of(PostfachNachrichtTestFactory.create()));
				when(mapper.toGrpc(any(PostfachNachricht.class))).thenReturn(GrpcRueckfrage.getDefaultInstance());
			}

			@Test
			void shouldCallMapper() {
				antragsraumGrpcService.findRueckfragen(GrpcFindRueckfrageRequestTestFactory.create(), streamObserver);

				verify(mapper).toGrpc(any(PostfachNachricht.class));
			}

			@Test
			void shouldCallOnNext() {
				antragsraumGrpcService.findRueckfragen(GrpcFindRueckfrageRequestTestFactory.create(), streamObserver);

				verify(streamObserver).onNext(any(GrpcFindRueckfragenResponse.class));
			}

			@Test
			void shouldCallOnCompleted() {
				antragsraumGrpcService.findRueckfragen(GrpcFindRueckfrageRequestTestFactory.create(), streamObserver);

				verify(streamObserver).onCompleted();
			}
		}
	}

	@Nested
	class TestSendAnswer {
		@Mock
		private StreamObserver<GrpcSendRueckfrageAnswerResponse> streamObserver;

		@BeforeEach
		void setup() {
			when(antragraumService.sendRueckfrageAnswer(anyString(), anyString(), any(PostfachNachricht.class)))
					.thenReturn(UUID.randomUUID().toString());
			when(mapper.toPostfachNachricht(any(GrpcRueckfrageAnswer.class))).thenReturn(PostfachNachrichtTestFactory.create());
		}

		@Test
		void shouldCallMapper() {
			antragsraumGrpcService.sendRueckfrageAnswer(GrpcSendRueckfrageAnswerRequestTestFactory.create(), streamObserver);

			verify(mapper).toPostfachNachricht(any(GrpcRueckfrageAnswer.class));
		}

		@Test
		void shouldCallAntragraumService() {
			antragsraumGrpcService.sendRueckfrageAnswer(GrpcSendRueckfrageAnswerRequestTestFactory.create(), streamObserver);

			verify(antragraumService).sendRueckfrageAnswer(anyString(), anyString(), any(PostfachNachricht.class));
		}

		@Test
		void shouldCallOnNext() {
			antragsraumGrpcService.sendRueckfrageAnswer(GrpcSendRueckfrageAnswerRequestTestFactory.create(), streamObserver);

			verify(streamObserver).onNext(any(GrpcSendRueckfrageAnswerResponse.class));
		}

		@Test
		void shouldCallOnCompleted() {
			antragsraumGrpcService.sendRueckfrageAnswer(GrpcSendRueckfrageAnswerRequestTestFactory.create(), streamObserver);

			verify(streamObserver).onCompleted();
		}

	}
}